@@ -16,8 +16,6 @@ import IPy
 import unittest
 import random
 
-testloops = 250
-
 class parseAddress(unittest.TestCase):
     okValues = [('FEDC:BA98:7654:3210:FEDC:BA98:7654:3210', 338770000845734292534325025077361652240),
                 ('FEDCBA9876543210FEDCBA9876543210', 338770000845734292534325025077361652240),
@@ -57,14 +55,14 @@ class parseAddress(unittest.TestCase):
                 ('256', 256),
                 ('0xffffffff', 4294967295),
                 ('0x100000000', 4294967296),
-                ('0xffffffffffffffffffffffffffffffff', 0xffffffffffffffffffffffffffffffffL),
-                ('0xdeadbeef', 0xdeadbeefL),
-                ('0xdeadbabe', 0xdeadbabeL),
-                ('0xdeadc0de', 0xdeadc0deL),
-                ('0xc0decafe', 0xc0decafeL),
-                ('0xc0debabe', 0xc0debabeL),
-                ('0xbabec0de', 0xbabec0deL),
-                ('0xcafebabe', 0xcafebabeL),
+                ('0xffffffffffffffffffffffffffffffff', 0xffffffffffffffffffffffffffffffff),
+                ('0xdeadbeef', 0xdeadbeef),
+                ('0xdeadbabe', 0xdeadbabe),
+                ('0xdeadc0de', 0xdeadc0de),
+                ('0xc0decafe', 0xc0decafe),
+                ('0xc0debabe', 0xc0debabe),
+                ('0xbabec0de', 0xbabec0de),
+                ('0xcafebabe', 0xcafebabe),
                 ('0x1', 1),
                 ('0xabcdef', 11259375)]
 
@@ -141,21 +139,26 @@ class parseAddress(unittest.TestCase):
         """'1111:2222:3333:4444:5555:6666:7777:8888:9999' should raise an exception"""
         self.assertRaises(ValueError, IPy.parseAddress, '1111:2222:3333:4444:5555:6666:7777:8888:9999')
 
+    def testBogusValues(self):
+        """Text values should raise an exception"""
+        self.assertRaises(ValueError, IPy.parseAddress, 'xx')
+        self.assertRaises(ValueError, IPy.parseAddress, 'foobar')
+
 class _intToIP(unittest.TestCase):
     v4values = [(0x7f000001, '127.0.0.1'),
                 (0x0, '0.0.0.0'),
                 (0x1, '0.0.0.1'),
                 (0xf, '0.0.0.15'),
                 (0xff, '0.0.0.255'),
-                (0xFFFFFFFFL, '255.255.255.255')]
+                (0xFFFFFFFF, '255.255.255.255')]
     v6values = [(0x7f000001, '0000:0000:0000:0000:0000:0000:7f00:0001'),
                 (0x0, '0000:0000:0000:0000:0000:0000:0000:0000'),
                 (0x1, '0000:0000:0000:0000:0000:0000:0000:0001'),
                 (0xf, '0000:0000:0000:0000:0000:0000:0000:000f'),
                 (0xff, '0000:0000:0000:0000:0000:0000:0000:00ff'),
-                (0xFFFFFFFFL, '0000:0000:0000:0000:0000:0000:ffff:ffff'),
+                (0xFFFFFFFF, '0000:0000:0000:0000:0000:0000:ffff:ffff'),
                 (0x100000000, '0000:0000:0000:0000:0000:0001:0000:0000'),
-                (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFL, 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff')]
+                (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF, 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff')]
 
     def testKnownValuesv4(self):
         """printing of known IPv4 values should give known results"""
@@ -197,21 +200,6 @@ class _intToIP(unittest.TestCase):
         self.assertRaises(ValueError, IPy.intToIp, 1, 7)
         self.assertRaises(ValueError, IPy.intToIp, 1, 8)
 
-class ParseAndBack(unittest.TestCase):
-    def testRandomValuesv4(self):
-        for i in range(testloops):
-            question = long(random.randrange(0x7fffffff)) + long(random.randrange(0x7fffffff))
-            self.assertEqual(IPy.parseAddress(IPy.intToIp(question, 4)), (question, 4), hex(question))
-
-    def testRandomValuesv6(self):
-        for i in range(testloops):
-            question = ((long(random.randrange(0x7fffffff)) + long(random.randrange(0x7fffffff))) +
-                        ((long(random.randrange(0x7fffffff)) + long(random.randrange(0x7fffffff))) << 32) +
-                        ((long(random.randrange(0x7fffffff)) + long(random.randrange(0x7fffffff))) << 64) +
-                        ((long(random.randrange(0x7fffffff)) + long(random.randrange(0x7fffffff))) << 96))
-            self.assertEqual(IPy.parseAddress(IPy.intToIp(question, 6)), (question, 6), hex(question))
-
-
 class _countXBits(unittest.TestCase):
     def testCount1Bits(self):
         self.assertEqual(IPy._count1Bits(0), 0)
@@ -219,10 +207,10 @@ class _countXBits(unittest.TestCase):
         self.assertEqual(IPy._count1Bits(0x10), 5)
         self.assertEqual(IPy._count1Bits(0xff), 8)
         self.assertEqual(IPy._count1Bits(0xffff), 16)
-        self.assertEqual(IPy._count1Bits(0xffffffffL), 32)
-        self.assertEqual(IPy._count1Bits(0xffffffffffffffffffffffffffffffffL), 128)
+        self.assertEqual(IPy._count1Bits(0xffffffff), 32)
+        self.assertEqual(IPy._count1Bits(0xffffffffffffffffffffffffffffffff), 128)
 
-    def testCount1Bits(self):
+    def testCount0Bits(self):
         self.assertEqual(IPy._count0Bits(0), 0)
         self.assertEqual(IPy._count0Bits(0xf0), 4)
         self.assertEqual(IPy._count0Bits(0xf00), 8)
@@ -242,9 +230,9 @@ class _intToBin(unittest.TestCase):
     knownValues = [(0, '0'), (1, '1'), (2, '10'), (3, '11'), (4, '100'), (5, '101'),
                    (6, '110'), (7, '111'), (8, '1000'), (9, '1001'),
                    (0xf, '1111'), (0xff, '11111111'),
-                   (0xFFFFFFFFL, '11111111111111111111111111111111'),
+                   (0xFFFFFFFF, '11111111111111111111111111111111'),
                    (0x100000000, '100000000000000000000000000000000'),
-                   (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFL, '11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111'),
+                   (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF, '11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111'),
                    (0x100000000000000000000000000000000, '100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000')]
 
     def testKnownValues(self):
@@ -259,7 +247,7 @@ class _intToBin(unittest.TestCase):
         self.assertRaises(ValueError, IPy._intToBin, -1)
 
 class netmaskPrefixlenConv(unittest.TestCase):
-    known4Values = [(0xFFFFFFFFL, 32), (0xFFFFFFFEL, 31), (0xFFFFFFFCL, 30), (0xFFFFFFF8, 29),
+    known4Values = [(0xFFFFFFFF, 32), (0xFFFFFFFE, 31), (0xFFFFFFFC, 30), (0xFFFFFFF8, 29),
                     (0xFFFFFFF0, 28), (0xFFFFFFE0, 27), (0xFFFFFFC0, 26), (0xFFFFFF80, 25),
                     (0xFFFFFF00, 24), (0xFFFFFE00, 23), (0xFFFFFC00, 22), (0xFFFFF800, 21),
                     (0xFFFFF000, 20), (0xFFFFE000, 19), (0xFFFFC000, 18), (0xFFFF8000, 17),
@@ -267,8 +255,8 @@ class netmaskPrefixlenConv(unittest.TestCase):
                     (0xFFF00000, 12), (0xFFE00000, 11), (0xFFC00000, 10), (0xFF800000, 9),
                     (0xFF000000, 8), (0xFE000000, 7), (0xFC000000, 6), (0xF8000000, 5),
                     (0xF0000000, 4), (0xE0000000, 3), (0xC0000000, 2), (0x80000000, 1)]
-    known6Values = [(0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFL, 128),
-                    (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEL, 127),
+    known6Values = [(0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF, 128),
+                    (0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE, 127),
                     (0xFFFFFFFFFFFFFFFFFFFFFFFF80000000, 97),
                     (0xFFFFFFFFFFFFFFFFFFFFFFFF00000000, 96),
                     (0xFFFFFFFFFFFFFFFFFFFFFFFE00000000, 95),
@@ -311,67 +299,67 @@ class netmaskPrefixlenConv(unittest.TestCase):
 
     def testInvalidv4n2p(self):
         """Netmasks should be all ones in the first part and all zeros in the second part"""
-        self.failUnlessRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00)
+        self.assertRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00)
 
     def testInvalidv6n2p(self):
         """Netmasks should be all ones in the first part and all zeros in the second part"""
-        self.failUnlessRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00ff00ff00ff00ff00ff00ff00)
+        self.assertRaises(ValueError, IPy._netmaskToPrefixlen, 0xff00ff00ff00ff00ff00ff00ff00ff00)
 
 
 class checkChecks(unittest.TestCase):
 
     def testCheckNetmaskOk(self):
         """Legal Netmasks should be allowed."""
-        self.failIf(IPy._checkNetmask(0xffffffffL, 32))
-        self.failIf(IPy._checkNetmask(0xffffff00, 32))
-        self.failIf(IPy._checkNetmask(0xffff0000, 32))
-        self.failIf(IPy._checkNetmask(0xff000000, 32))
-        self.failIf(IPy._checkNetmask(0, 32))
+        self.assertFalse(IPy._checkNetmask(0xffffffff, 32))
+        self.assertFalse(IPy._checkNetmask(0xffffff00, 32))
+        self.assertFalse(IPy._checkNetmask(0xffff0000, 32))
+        self.assertFalse(IPy._checkNetmask(0xff000000, 32))
+        self.assertFalse(IPy._checkNetmask(0, 32))
 
     def testCheckNetmaskFail(self):
         """Illegal Netmasks should be rejected."""
-        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xf0ffffffL, 32)
-        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xf0f0f0f0, 32)
-        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xff00ff00, 32)
-        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0x70000001, 32)
-        self.failUnlessRaises(ValueError, IPy._checkNetmask, 0xfffffffL, 32)
+        self.assertRaises(ValueError, IPy._checkNetmask, 0xf0ffffff, 32)
+        self.assertRaises(ValueError, IPy._checkNetmask, 0xf0f0f0f0, 32)
+        self.assertRaises(ValueError, IPy._checkNetmask, 0xff00ff00, 32)
+        self.assertRaises(ValueError, IPy._checkNetmask, 0x70000001, 32)
+        self.assertRaises(ValueError, IPy._checkNetmask, 0xfffffff, 32)
 
     def testCheckPrefixOk(self):
         """Legal IP/prefix combinations should check ok."""
-        self.failUnless(IPy._checkPrefix(0x0, 32, 4))
-        self.failUnless(IPy._checkPrefix(0xffffffffL, 32, 4))
-        self.failUnless(IPy._checkPrefix(0x7f000001, 32, 4))
-        self.failUnless(IPy._checkPrefix(0x80000000, 1, 4))
-        self.failUnless(IPy._checkPrefix(0x40000000, 2, 4))
-        self.failUnless(IPy._checkPrefix(0x80000000, 3, 4))
-        self.failUnless(IPy._checkPrefix(0x80000000, 4, 4))
-        self.failUnless(IPy._checkPrefix(0xffffff00, 24, 4))
-        self.failUnless(IPy._checkPrefix(0xffffff00, 24, 4))
-        self.failUnless(IPy._checkPrefix(0xfffffff0, 28, 4))
-        self.failUnless(IPy._checkPrefix(0x0, 32, 4))
-        self.failUnless(IPy._checkPrefix(0x0, 1, 4))
-        self.failUnless(IPy._checkPrefix(0x0, 0, 4))
-        self.failUnless(IPy._checkPrefix(0xffffffffffffffff0000000000000000, 64, 6))
-        self.failUnless(IPy._checkPrefix(0x0, 64, 6))
-        self.failUnless(IPy._checkPrefix(0x0, 0, 6))
-        self.failUnless(IPy._checkPrefix(0x0, 128, 6))
-        self.failUnless(IPy._checkPrefix(0xffffffffffffffffffffffffffffffffL, 128, 6))
+        self.assertTrue(IPy._checkPrefix(0x0, 32, 4))
+        self.assertTrue(IPy._checkPrefix(0xffffffff, 32, 4))
+        self.assertTrue(IPy._checkPrefix(0x7f000001, 32, 4))
+        self.assertTrue(IPy._checkPrefix(0x80000000, 1, 4))
+        self.assertTrue(IPy._checkPrefix(0x40000000, 2, 4))
+        self.assertTrue(IPy._checkPrefix(0x80000000, 3, 4))
+        self.assertTrue(IPy._checkPrefix(0x80000000, 4, 4))
+        self.assertTrue(IPy._checkPrefix(0xffffff00, 24, 4))
+        self.assertTrue(IPy._checkPrefix(0xffffff00, 24, 4))
+        self.assertTrue(IPy._checkPrefix(0xfffffff0, 28, 4))
+        self.assertTrue(IPy._checkPrefix(0x0, 32, 4))
+        self.assertTrue(IPy._checkPrefix(0x0, 1, 4))
+        self.assertTrue(IPy._checkPrefix(0x0, 0, 4))
+        self.assertTrue(IPy._checkPrefix(0xffffffffffffffff0000000000000000, 64, 6))
+        self.assertTrue(IPy._checkPrefix(0x0, 64, 6))
+        self.assertTrue(IPy._checkPrefix(0x0, 0, 6))
+        self.assertTrue(IPy._checkPrefix(0x0, 128, 6))
+        self.assertTrue(IPy._checkPrefix(0xffffffffffffffffffffffffffffffff, 128, 6))
 
 
     def testCheckPrefixFail(self):
         """Illegal Prefixes should be catched."""
-        self.failIf(IPy._checkPrefix(0x7f000001, -1, 4))
-        self.failIf(IPy._checkPrefix(0x7f000001, 33, 4))
-        self.failIf(IPy._checkPrefix(0x7f000001, 24, 4))
-        self.failIf(IPy._checkPrefix(0x7f000001, 31, 4))
-        self.failIf(IPy._checkPrefix(0x7f000080, 24, 4))
-        self.failIf(IPy._checkPrefix(0x7f000100, 23, 4))
-        self.failIf(IPy._checkPrefix(0x7f000000, 1, 4))
-        self.failIf(IPy._checkPrefix(0x7f000000, 0, 4))
-        self.failIf(IPy._checkPrefix(0x1, -1, 6))
-        self.failIf(IPy._checkPrefix(0x1, 129, 6))
-        self.failIf(IPy._checkPrefix(0xffffffffffffffff0000000000000001, 64, 6))
-        self.failIf(IPy._checkPrefix(0xffffffffffffffff1000000000000000, 64, 6))
+        self.assertFalse(IPy._checkPrefix(0x7f000001, -1, 4))
+        self.assertFalse(IPy._checkPrefix(0x7f000001, 33, 4))
+        self.assertFalse(IPy._checkPrefix(0x7f000001, 24, 4))
+        self.assertFalse(IPy._checkPrefix(0x7f000001, 31, 4))
+        self.assertFalse(IPy._checkPrefix(0x7f000080, 24, 4))
+        self.assertFalse(IPy._checkPrefix(0x7f000100, 23, 4))
+        self.assertFalse(IPy._checkPrefix(0x7f000000, 1, 4))
+        self.assertFalse(IPy._checkPrefix(0x7f000000, 0, 4))
+        self.assertFalse(IPy._checkPrefix(0x1, -1, 6))
+        self.assertFalse(IPy._checkPrefix(0x1, 129, 6))
+        self.assertFalse(IPy._checkPrefix(0xffffffffffffffff0000000000000001, 64, 6))
+        self.assertFalse(IPy._checkPrefix(0xffffffffffffffff1000000000000000, 64, 6))
 
 
     # TODO: _checkNetaddrWorksWithPrefixlen(net, prefixlen, version):
@@ -422,61 +410,69 @@ class PythonObjectBehaviour(unittest.TestCase):
         """__contains__() should work somewhat with ints"""
         ip = IPy.IP('127.0.0.0/28')
         for x in ip:
-            self.failUnless(x.int() in ip)
+            self.assertTrue(x.int() in ip)
         ip = IPy.IP('2001::/124')
         for x in ip:
-            self.failUnless(x.int() in ip)
+            self.assertTrue(x.int() in ip)
 
     def testIfContainsStr(self):
         """__contains__() should work somewhat with strings"""
         ip = IPy.IP('127.0.0.0/28')
         for x in ip:
-            self.failUnless(x.strNormal() in ip, "%r not in %r" % (x.strNormal(), ip))
+            self.assertTrue(x.strNormal() in ip, "%r not in %r" % (x.strNormal(), ip))
         ip = IPy.IP('2001::/124')
         for x in ip:
-            self.failUnless(x.strNormal() in ip, "%r not in %r" % (x.strNormal(), ip))
+            self.assertTrue(x.strNormal() in ip, "%r not in %r" % (x.strNormal(), ip))
 
     def testIfContainsIPobj(self):
         """__contains__() should work somewhat with IP instances"""
         ip = IPy.IP('127.0.0.0/28')
         for x in ip:
-            self.failUnless(x in ip)
+            self.assertTrue(x in ip)
         ip = IPy.IP('2001::/124')
         for x in ip:
-            self.failUnless(x in ip)
+            self.assertTrue(x in ip)
+
+    def testContainsVersionSeparation(self):
+        """__contains__() should return false if versions mismatch"""
+        four = IPy.IP('192.168.0.0/16')
+        six = IPy.IP('::c0a8:0/112')
+        self.assertFalse(four in six)
+        self.assertFalse(six in four)
 
     def testActingAsArray(self):
         """An IP-object should handle indices."""
         ip = IPy.IP('127.0.0.0/24')
         self.assertEqual(ip[0], ip.net())
         self.assertEqual(ip[-1], ip.broadcast())
-        self.failUnless(ip[255])
-        self.failUnlessRaises(IndexError, ip.__getitem__, 256)
+        self.assertTrue(ip[255])
+        self.assertTrue(isinstance(ip[4::4], list))
+        self.assertRaises(IndexError, ip.__getitem__, 256)
 
     def testStr(self):
         """string() should work somewhat with IP instances"""
         ip = IPy.IP('127.0.0.0/28')
         for x in ip:
-            self.failUnless(str(x))
+            self.assertTrue(str(x))
         ip = IPy.IP('2001::/124')
         for x in ip:
-            self.failUnless(str(x))
+            self.assertTrue(str(x))
 
     def testRepr(self):
         """repr() should work somewhat with IP instances"""
         ip = IPy.IP('127.0.0.0/28')
         for x in ip:
-            self.failUnless(repr(x))
+            self.assertTrue(repr(x))
         ip = IPy.IP('2001::/124')
         for x in ip:
-            self.failUnless(repr(x))
+            self.assertTrue(repr(x))
 
     def testLen(self):
         """object should have an working __len__() interface."""
-        self.failUnlessEqual(len(IPy.IP('127.0.0.0/28')), 16)
-        self.failUnlessEqual(len(IPy.IP('127.0.0.0/30')), 4)
-        self.failUnlessEqual(len(IPy.IP('127.0.0.0/26')), 64)
-        self.failUnlessEqual(len(IPy.IP('127.0.0.0/16')), 2**16)
+        self.assertEqual(len(IPy.IP('127.0.0.0/28')), 16)
+        self.assertEqual(len(IPy.IP('127.0.0.0/30')), 4)
+        self.assertEqual(len(IPy.IP('127.0.0.0/26')), 64)
+        self.assertEqual(len(IPy.IP('127.0.0.0/16')), 2**16)
 
     # cmp
     # IP[0xffffffff]
@@ -512,7 +508,7 @@ class IPobject(unittest.TestCase):
                   '1::2:0:0:3']
         for question in testValues:
             result = IPy.IP(question).strCompressed()
-            self.failUnlessEqual(question, result, (question, result))
+            self.assertEqual(question, result, (question, result))
 
     def testStrBin(self):
         """Binary string Output."""
@@ -532,7 +528,7 @@ class IPobject(unittest.TestCase):
                       ('1::2:0:0:3', '00000000000000010000000000000000000000000000000000000000000000000000000000000010000000000000000000000000000000000000000000000011')]
         for (question, answer) in testValues:
             result = IPy.IP(question).strBin()
-            self.failUnlessEqual(answer, result, (question, answer, result))
+            self.assertEqual(answer, result, (question, answer, result))
 
     def testStrNormal(self):
         """Normal string Output."""
@@ -546,7 +542,7 @@ class IPobject(unittest.TestCase):
                       (3588059479, '213.221.113.87')]
         for (question, answer) in testValues:
             result = IPy.IP(question).strNormal(question)
-            self.failUnlessEqual(answer, result, (question, result, answer))
+            self.assertEqual(answer, result, (question, result, answer))
 
     def testStrFullsize(self):
         """Normal / 0-padded string Output."""
@@ -560,7 +556,7 @@ class IPobject(unittest.TestCase):
                       (3588059479, '213.221.113.87')]
         for (question, answer) in testValues:
             result = IPy.IP(question).strFullsize(question)
-            self.failUnlessEqual(answer, result, (question, result, answer))
+            self.assertEqual(answer, result, (question, result, answer))
 
     def testStrHex(self):
         """Hex string Output."""
@@ -569,12 +565,12 @@ class IPobject(unittest.TestCase):
                       (338958331222012082418099330867817087043, '0xff010000000000000000000000000043'),
                       (0, '0x0'),
                       (1, '0x1'),
-                      (4294967295l, '0xffffffff'),
+                      (4294967295, '0xffffffff'),
                       (3588059479, '0xd5dd7157'),
                       (0x12345678, '0x12345678')]
         for (question, answer) in testValues:
             result = IPy.IP(question).strHex(question).lower()
-            self.failUnlessEqual(answer, result, (question, result, answer))
+            self.assertEqual(answer, result, (question, result, answer))
 
     def testStrDec(self):
         """Decimal string Output."""
@@ -583,65 +579,65 @@ class IPobject(unittest.TestCase):
                       (338958331222012082418099330867817087043, '338958331222012082418099330867817087043'),
                       (0, '0'),
                       (1, '1'),
-                      (0xFFFFFFFFL, '4294967295'),
+                      (0xFFFFFFFF, '4294967295'),
                       (0xD5DD7157, '3588059479')]
         for (question, answer) in testValues:
             result = IPy.IP(question).strDec(question)
-            self.failUnlessEqual(answer, result, (question, result, answer))
+            self.assertEqual(answer, result, (question, result, answer))
 
     def testNet(self):
         """Returning of the Network Address"""
-        self.failUnlessEqual(str(IPy.IP("127.0.0.1").net()), "127.0.0.1")
-        self.failUnlessEqual(str(IPy.IP("0.0.0.0/0").net()), "0.0.0.0")
-        self.failUnlessEqual(str(IPy.IP("2001:1234:5678:1234::/64").net()), "2001:1234:5678:1234::")
+        self.assertEqual(str(IPy.IP("127.0.0.1").net()), "127.0.0.1")
+        self.assertEqual(str(IPy.IP("0.0.0.0/0").net()), "0.0.0.0")
+        self.assertEqual(str(IPy.IP("2001:1234:5678:1234::/64").net()), "2001:1234:5678:1234::")
 
 
     def testBroadcast(self):
         """Returning of broadcast address."""
-        self.failUnlessEqual(str(IPy.IP("127.0.0.1").broadcast()), "127.0.0.1")
-        self.failUnlessEqual(str(IPy.IP("0.0.0.0/0").broadcast()), "255.255.255.255")
-        self.failUnlessEqual(str(IPy.IP("2001:1234:5678:1234::/64").broadcast()), "2001:1234:5678:1234:ffff:ffff:ffff:ffff")
+        self.assertEqual(str(IPy.IP("127.0.0.1").broadcast()), "127.0.0.1")
+        self.assertEqual(str(IPy.IP("0.0.0.0/0").broadcast()), "255.255.255.255")
+        self.assertEqual(str(IPy.IP("2001:1234:5678:1234::/64").broadcast()), "2001:1234:5678:1234:ffff:ffff:ffff:ffff")
 
 
     def testStrNetmask(self):
         """StrNetmask should return netmasks"""
-        self.failUnlessEqual(IPy.IP("0.0.0.0/0").strNetmask(), "0.0.0.0")
-        self.failUnlessEqual(IPy.IP("0.0.0.0/32").strNetmask(), "255.255.255.255")
-        self.failUnlessEqual(IPy.IP("127.0.0.0/24").strNetmask(), "255.255.255.0")
-        self.failUnlessEqual(IPy.IP("2001:1234:5678:1234::/64").strNetmask(), "/64")
+        self.assertEqual(IPy.IP("0.0.0.0/0").strNetmask(), "0.0.0.0")
+        self.assertEqual(IPy.IP("0.0.0.0/32").strNetmask(), "255.255.255.255")
+        self.assertEqual(IPy.IP("127.0.0.0/24").strNetmask(), "255.255.255.0")
+        self.assertEqual(IPy.IP("2001:1234:5678:1234::/64").strNetmask(), "/64")
 
 
     def testNetmask(self):
         """Netmask should return netmasks"""
-        self.failUnlessEqual(str(IPy.IP("0.0.0.0/0").netmask()), "0.0.0.0")
-        self.failUnlessEqual(str(IPy.IP("0.0.0.0/32").netmask()), "255.255.255.255")
-        self.failUnlessEqual(str(IPy.IP("127.0.0.0/24").netmask()), "255.255.255.0")
-        self.failUnlessEqual(str(IPy.IP("2001:1234:5678:1234::/64").netmask()), "ffff:ffff:ffff:ffff:0000:0000:0000:0000")
+        self.assertEqual(str(IPy.IP("0.0.0.0/0").netmask()), "0.0.0.0")
+        self.assertEqual(str(IPy.IP("0.0.0.0/32").netmask()), "255.255.255.255")
+        self.assertEqual(str(IPy.IP("127.0.0.0/24").netmask()), "255.255.255.0")
+        self.assertEqual(str(IPy.IP("2001:1234:5678:1234::/64").netmask()), "ffff:ffff:ffff:ffff::")
 
     def testInt(self):
         """Prefixlen"""
-        self.failUnlessEqual(IPy.IP("127.0.0.1").int(), 2130706433)
-        self.failUnlessEqual(IPy.IP("0.0.0.0").int(), 0)
-        self.failUnlessEqual(IPy.IP("255.255.255.255").int(), 0xffffffffL)
-        self.failUnlessEqual(IPy.IP("0000:0000:0000:0000:0000:0000:0000:0000").int(), 0)
-        self.failUnlessEqual(IPy.IP("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff").int(), 0xffffffffffffffffffffffffffffffffL)
-        self.failUnlessEqual(IPy.IP("2001:1234:5678:9abc:de00:0000:0000:0000").int(), 42540857391974671903776007410583339008)
+        self.assertEqual(IPy.IP("127.0.0.1").int(), 2130706433)
+        self.assertEqual(IPy.IP("0.0.0.0").int(), 0)
+        self.assertEqual(IPy.IP("255.255.255.255").int(), 0xffffffff)
+        self.assertEqual(IPy.IP("0000:0000:0000:0000:0000:0000:0000:0000").int(), 0)
+        self.assertEqual(IPy.IP("ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff").int(), 0xffffffffffffffffffffffffffffffff)
+        self.assertEqual(IPy.IP("2001:1234:5678:9abc:de00:0000:0000:0000").int(), 42540857391974671903776007410583339008)
 
 
     def testPrefixlen(self):
         """Prefixlen"""
-        self.failUnlessEqual(IPy.IP("127.0.0.1").prefixlen(), 32)
-        self.failUnlessEqual(IPy.IP("::1").prefixlen(), 128)
-        self.failUnlessEqual(IPy.IP("10.0.0.0/24").prefixlen(), 24)
-        self.failUnlessEqual(IPy.IP("10.0.0.0-10.0.0.255").prefixlen(), 24)
-        self.failUnlessEqual(IPy.IP("10.0.0.0/255.255.255.0").prefixlen(), 24)
-        self.failUnlessEqual(IPy.IP("2001::/64").prefixlen(), 64)
+        self.assertEqual(IPy.IP("127.0.0.1").prefixlen(), 32)
+        self.assertEqual(IPy.IP("::1").prefixlen(), 128)
+        self.assertEqual(IPy.IP("10.0.0.0/24").prefixlen(), 24)
+        self.assertEqual(IPy.IP("10.0.0.0-10.0.0.255").prefixlen(), 24)
+        self.assertEqual(IPy.IP("10.0.0.0/255.255.255.0").prefixlen(), 24)
+        self.assertEqual(IPy.IP("2001::/64").prefixlen(), 64)
 
 
     def testVersion(self):
         """IP-version detection should work"""
-        self.failUnlessEqual(IPy.IP("0.0.0.0/0").version(), 4)
-        self.failUnlessEqual(IPy.IP("::1").version(), 6)
+        self.assertEqual(IPy.IP("0.0.0.0/0").version(), 4)
+        self.assertEqual(IPy.IP("::1").version(), 6)
 
     # TODO:
     #def reverseNames(self):
@@ -663,20 +659,15 @@ class IPobject(unittest.TestCase):
                       ('127.0.0.1', '127.0.0.2', 0)]
         for (a, b, answer) in testValues:
             result = IPy.IP(a).overlaps(b)
-            self.failUnlessEqual(answer, result, (a, b, result, answer))
+            self.assertEqual(answer, result, (a, b, result, answer))
 
-    def testNetmask(self):
-        """Normal string Output."""
-        testValues = [(338770000845734292534325025077361652240, '0xfedcba9876543210fedcba9876543210'),
-                      (21932261930451111902915077091070067066, '0x108000000000000000080800200c417a'),
-                      (338958331222012082418099330867817087043, '0xff010000000000000000000000000043'),
-                      (0, '0x0'),
-                      (1, '0x1'),
-                      (4294967295l, '0xffffffff'),
-                      (3588059479, '0xd5dd7157')]
-        for (question, answer) in testValues:
-            result = IPy.IP(question).strHex(question).lower()
-            self.failUnlessEqual(answer, result, (question, result, answer))
+    def testV46map(self):
+        four    = IPy.IP('192.168.1.1')
+        six     = IPy.IP('::ffff:192.168.1.1')
+        invalid = IPy.IP('2001::ffff:192.168.1.1')
+        self.assertEqual(four.v46map(), six)
+        self.assertEqual(four, six.v46map())
+        self.assertRaises(ValueError, invalid.v46map)
 
 # TODO
 #eval(repr(IPy))
@@ -705,7 +696,7 @@ class NetIPChecks(unittest.TestCase):
         ip = IPy.IP('202.31.4/24')
         self.assertEqual(str(ip.net()),'202.31.4.0')
 
-        self.failUnlessRaises(ValueError, IPy.IP, '234.245.252.253/2')
+        self.assertRaises(ValueError, IPy.IP, '234.245.252.253/2')
 
         # because we ar using integer representation we don't need a special "binadd"
         ip = IPy.IP('62.33.41.9')
@@ -715,7 +706,12 @@ class NetIPChecks(unittest.TestCase):
 
         ip = IPy.IP('133.45.0/24')
         ip2 = IPy.IP('133.45.1/24')
+        ip3 = IPy.IP('133.45.2/24')
         self.assertEqual((ip + ip2).prefixlen(),23)
+        # Non-adjacent ranges
+        self.assertRaises(ValueError, IPy.IP.__add__, ip, ip3)
+        # Resulting invalid prefix
+        self.assertRaises(ValueError, IPy.IP.__add__, ip2, ip3)
 
         ip2 = IPy.IP('133.44.255.255');
         #$T->ok_eqnum ($ip->bincomp('gt',$ip2),1,$ip->error());
@@ -756,6 +752,7 @@ class NetIPChecks(unittest.TestCase):
         ip = IPy.IP('::e000:0/112')
         ip2 = IPy.IP('::e001:0/112')
         self.assertEqual(ip.__add__(ip2).prefixlen(),111)
+        self.assertEqual(ip.__add__(ip2).version(),6)
 
         ip2 = IPy.IP('::dfff:ffff')
         #$T->ok_eqnum ($ip->bincomp('gt',$ip2),1,$ip->error());
@@ -791,11 +788,84 @@ def timeout(func, args=(), kwargs={}, timeout_duration=1, default=None):
     it.setDaemon(True)
     it.start()
     it.join(timeout_duration)
-    if it.isAlive():
+    if hasattr(it, 'is_alive'):
+        is_alive = it.is_alive()
+    else:
+        is_alive = it.isAlive()
+    if is_alive:
         return default
     else:
         return it.result
 
+class IPSetChecks(unittest.TestCase):
+    def setUp(self):
+        #array
+        self.a = [IPy.IP("192.168." + str(i) + ".0/24") for i in range(256)]
+        #range
+        self.r = IPy.IP('192.168.0.0/16')
+        #testing set
+        self.t = IPy.IPSet(self.a)
+        #control set
+        self.c = IPy.IPSet(self.a)
+        #Could otherwise look like 192.168.128.0/17
+        self.sixRange = IPy.IP('::c0a8:8000/113')
+
+    def testVersionSeparation(self):
+        #Don't remove a matching IPv6 subnet from an IPv4 list
+        self.assertRaises(KeyError, self.t.remove, self.sixRange)
+        self.t.add(self.sixRange)
+        self.assertNotEqual(self.t, self.c)
+        self.t.remove(self.sixRange)
+        self.t.discard(self.sixRange)
+        self.assertEqual(self.t, self.c)
+
+    def testAnd(self):
+        ten24s = IPy.IPSet([
+            IPy.IP('10.0.1.0/24'),
+            IPy.IP('10.0.3.0/24'),
+            IPy.IP('10.0.5.0/24'),
+            IPy.IP('10.0.7.0/24'),
+        ])
+
+        self.assertEqual(ten24s & IPy.IPSet([IPy.IP('10.0.1.10')]),
+                         IPy.IPSet([IPy.IP('10.0.1.10')]))
+
+        self.assertEqual(ten24s & IPy.IPSet([
+            IPy.IP('10.0.0.99'),
+            IPy.IP('10.0.1.10'),
+            IPy.IP('10.0.3.40'),
+            IPy.IP('11.1.1.99'),
+        ]), IPy.IPSet([
+            IPy.IP('10.0.1.10'),
+            IPy.IP('10.0.3.40'),
+        ]))
+
+    def testContains(self):
+        self.assertTrue(IPy.IP('192.168.15.32/28') in self.t)
+        self.assertFalse(IPy.IP('192.169.15.32/28') in self.t)
+
+        # test for a regression where __contains__ prematurely returns False
+        # after testing a prefix length where all IP instances are greater than
+        # the query IP.
+        ipset = IPy.IPSet([IPy.IP('10.0.0.0/8'), IPy.IP('128.0.0.0/1')])
+        self.assertTrue(IPy.IP('10.0.0.0') in ipset)
+
+    def testIsdisjoint(self):
+        self.assertTrue(IPy.IPSet([IPy.IP('0.0.0.0/1')])
+                .isdisjoint(IPy.IPSet([IPy.IP('128.0.0.0/1')])))
+        self.assertFalse(IPy.IPSet([IPy.IP('0.0.0.0/1')])
+                .isdisjoint(IPy.IPSet([IPy.IP('0.0.0.0/2')])))
+        self.assertFalse(IPy.IPSet([IPy.IP('0.0.0.0/2')])
+                .isdisjoint(IPy.IPSet([IPy.IP('0.0.0.0/1')])))
+        self.assertFalse(IPy.IPSet([IPy.IP('0.0.0.0/2')])
+                .isdisjoint(IPy.IPSet([IPy.IP('0.1.2.3')])))
+        self.assertFalse(IPy.IPSet([IPy.IP('0.1.2.3')])
+                .isdisjoint(IPy.IPSet([IPy.IP('0.0.0.0/2')])))
+        self.assertTrue(IPy.IPSet([IPy.IP('1.1.1.1'), IPy.IP('1.1.1.3')])
+                .isdisjoint(IPy.IPSet([IPy.IP('1.1.1.2'), IPy.IP('1.1.1.4')])))
+        self.assertFalse(IPy.IPSet([IPy.IP('1.1.1.1'), IPy.IP('1.1.1.3'), IPy.IP('1.1.2.0/24')])
+                .isdisjoint(IPy.IPSet([IPy.IP('1.1.2.2'), IPy.IP('1.1.1.4')])))
+
 class RegressionTest(unittest.TestCase):
     def testNulNetmask(self):
         ip = timeout(IPy.IP, ["0.0.0.0/0.0.0.0"], timeout_duration=0.250, default=None)
@@ -825,6 +895,9 @@ class RegressionTest(unittest.TestCase):
         self.assertEqual(len(IPy.IP('192.168.0.0/24')), 256)
         self.assertRaises(ValueError, IPy.IP, '192.168.1.0/42')
 
+    def testConsistentIP6StrInt(self):
+        self.assertEqual(IPy.IP('11', ipversion=6), IPy.IP(11, ipversion=6))
+
 class TestConstrutor(unittest.TestCase):
     def testCheckAddrPrefixlenOff(self):
         self.assertRaises(ValueError, IPy.IP, 0xffffffff + 1, ipversion=4)
